class ReminderEntriesController < ApplicationController
  unloadable
  before_filter :find_project_by_project_id ,:authorize
  include ReminderEntriesHelper

  def index
    @project = Project.find(params[:project_id])
    @reminder_entries = ReminderEntry.where(project_id: @project.id)
  end

  def new
    @project = Project.find(params[:project_id])
    @reminder_entry = ReminderEntry.new

    @project_member_ids = @project.users.collect{|u| u.id}

    @useroptions = Array.new
    @project_member_ids.each do |project_member_id|
      user = User.find_by_id(project_member_id)
      name = user.firstname + " " + user.lastname
      @useroptions.push([name, project_member_id])
    end

    @envoptions = Array.new
    @envoptions.push(["Production", "production"])
    @envoptions.push(["Development", "development"])
    @envoptions.push(["Test", "test"])

    @project_tracker_ids = @project.trackers.collect{|u| u.id}
    @trackeroptions = Array.new
    @project_tracker_ids.each do |project_tracker_id|
      tracker = Tracker.find_by_id(project_tracker_id)
      @trackeroptions.push([tracker.name, project_tracker_id])
    end
  end

  def create
    @project = Project.find(params[:project_id])
    if params[:reminder][:tracker] == ""
      tracker_id = 0
    else
      tracker_id = params[:reminder][:tracker]
    end
    @reminder_entry = ReminderEntry.new(:project_id => @project.id,
                                        :tracker_id => tracker_id,
                                        :days => params[:reminder][:days],
                                        :env => params[:reminder][:env])

    if @reminder_entry.save
      @reminder_users = Array.new
      user_ids = params[:reminder][:users]
      user_ids.each do |user_id|
        unless user_id == ""
          reminder_user = ReminderUser.new(:reminder_entry_id => @reminder_entry.id,
                                           :user_id => user_id)
          @reminder_users.push(reminder_user)
        end
      end

      @reminder_users.each do |reminder_user|
        reminder_user.save
      end
      flash[:notice] = "Reminder set."
      forceUpdateScript
      redirect_to project_reminder_entries_path(:project_id => @project.id)
    else
      flash[:error] = "Reminder couldn't set! Please check the form."
      Rails.logger.info(@reminder_entry.errors.inspect)
      render new_project_reminder_entry_path(:project_id => @project.id)
    end
  end

  def destroy
    @project = Project.find(params[:project_id])
    @reminder_entry = ReminderEntry.find(params[:id])
    @status = @reminder_entry.destroy

    if @status
      @reminder_users = ReminderUser.where(reminder_entry_id: @reminder_entry.id)
      @reminder_users.each do |reminder_user|
        reminder_user.destroy
      end

      flash[:notice] = "Reminder removed."
      forceUpdateScript
      redirect_to project_reminder_entries_path
    else
      flash[:error] = "Reminder couldn't be removed!"
      render project_reminder_entries_path
    end
  end

  def send_now
    @project = Project.find(params[:project_id])
    @reminder_entry = ReminderEntry.find(params[:reminder_entry_id])

    options = {}
    options[:project] = @project.id
    options[:tracker] = @reminder_entry.tracker_id.to_i if @reminder_entry.tracker_id != 0
    options[:days] = @reminder_entry.days.to_i
    options[:users] = Array.new

    @reminder_users = ReminderUser.where(:reminder_entry_id => @reminder_entry.id)
    unless @reminder_users.length == 0
      @reminder_users.each do |reminder_user|
        options[:users].push(reminder_user.user_id)
      end
    end

    if send_reminders(options)
      flash[:notice] = "Mail sent."
      redirect_to project_reminder_entries_path(:project_id => @project.id)
    else
      flash[:error] = "Mail couldn't send!"
      render project_reminder_entries_path
    end
  end

  private
  def forceUpdateScript
    reminder_script = File.open(Setting['plugin_reminderemails']['script_path'], "w")
    File.chmod(0775, Setting['plugin_reminderemails']['script_path'])
    reminder_script.write(generateScriptContent)
    reminder_script.close
  end

  def generateScriptContent
    content = "#!/bin/bash\n"
    content << "cd " + Rails.root.to_s + "\n"
    rake_content = Setting['plugin_reminderemails']['rake_path'] + " redmine:send_reminders "

    reminder_entries = ReminderEntry.all
    reminder_entries.each do |reminder_entry|
      content << rake_content
      content << generateReminderOptions(reminder_entry)
      content << "\n"
    end

    return content
  end

  def generateReminderOptions(reminder_entry)
    options = "project=\"" + reminder_entry.project_id.to_s + "\" "
    options << "days=\"" + reminder_entry.days.to_s + "\" "
    options << "RAILS_ENV=\"" + reminder_entry.env + "\" "
    unless reminder_entry.tracker_id == 0
      options << "tracker=\"" + reminder_entry.tracker_id.to_s + "\" "
    end

    reminder_users = ReminderUser.where(:reminder_entry_id => reminder_entry.id)
    unless reminder_users.length == 0
      options << "users=\""
      reminder_users.each do |reminder_user|
        options << reminder_user.user_id.to_s + ","
      end
      options = options[0...-1]
      options << "\""
    end

    return options
  end

  def send_reminders(options)
    Mailer.with_synched_deliveries do
      Mailer.reminders(options)
    end

    return true
  end

end
